# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .maintenance_configs import (
    MaintenanceConfigsResource,
    AsyncMaintenanceConfigsResource,
    MaintenanceConfigsResourceWithRawResponse,
    AsyncMaintenanceConfigsResourceWithRawResponse,
    MaintenanceConfigsResourceWithStreamingResponse,
    AsyncMaintenanceConfigsResourceWithStreamingResponse,
)
from .....types.r2_data_catalog.namespaces import table_list_params
from .....types.r2_data_catalog.namespaces.table_list_response import TableListResponse

__all__ = ["TablesResource", "AsyncTablesResource"]


class TablesResource(SyncAPIResource):
    @cached_property
    def maintenance_configs(self) -> MaintenanceConfigsResource:
        return MaintenanceConfigsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TablesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TablesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TablesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TablesResourceWithStreamingResponse(self)

    def list(
        self,
        namespace: str,
        *,
        account_id: str,
        bucket_name: str,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        return_details: bool | Omit = omit,
        return_uuids: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TableListResponse]:
        """
        Returns a list of tables in the specified namespace within an R2 catalog.
        Supports pagination for efficient traversal of large table collections.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          page_size: Maximum number of tables to return per page. Defaults to 100, maximum 1000.

          page_token: Opaque pagination token from a previous response. Use this to fetch the next
              page of results.

          return_details: Whether to include additional metadata (timestamps, locations). When true,
              response includes created_at, updated_at, metadata_locations, and locations
              arrays.

          return_uuids: Whether to include table UUIDs in the response. Set to true to receive the
              table_uuids array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not namespace:
            raise ValueError(f"Expected a non-empty value for `namespace` but received {namespace!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/namespaces/{namespace}/tables",
                account_id=account_id,
                bucket_name=bucket_name,
                namespace=namespace,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_size": page_size,
                        "page_token": page_token,
                        "return_details": return_details,
                        "return_uuids": return_uuids,
                    },
                    table_list_params.TableListParams,
                ),
                post_parser=ResultWrapper[Optional[TableListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TableListResponse]], ResultWrapper[TableListResponse]),
        )


class AsyncTablesResource(AsyncAPIResource):
    @cached_property
    def maintenance_configs(self) -> AsyncMaintenanceConfigsResource:
        return AsyncMaintenanceConfigsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTablesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTablesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTablesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTablesResourceWithStreamingResponse(self)

    async def list(
        self,
        namespace: str,
        *,
        account_id: str,
        bucket_name: str,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        return_details: bool | Omit = omit,
        return_uuids: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TableListResponse]:
        """
        Returns a list of tables in the specified namespace within an R2 catalog.
        Supports pagination for efficient traversal of large table collections.

        Args:
          account_id: Use this to identify the account.

          bucket_name: Specifies the R2 bucket name.

          page_size: Maximum number of tables to return per page. Defaults to 100, maximum 1000.

          page_token: Opaque pagination token from a previous response. Use this to fetch the next
              page of results.

          return_details: Whether to include additional metadata (timestamps, locations). When true,
              response includes created_at, updated_at, metadata_locations, and locations
              arrays.

          return_uuids: Whether to include table UUIDs in the response. Set to true to receive the
              table_uuids array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        if not namespace:
            raise ValueError(f"Expected a non-empty value for `namespace` but received {namespace!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/r2-catalog/{bucket_name}/namespaces/{namespace}/tables",
                account_id=account_id,
                bucket_name=bucket_name,
                namespace=namespace,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page_size": page_size,
                        "page_token": page_token,
                        "return_details": return_details,
                        "return_uuids": return_uuids,
                    },
                    table_list_params.TableListParams,
                ),
                post_parser=ResultWrapper[Optional[TableListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TableListResponse]], ResultWrapper[TableListResponse]),
        )


class TablesResourceWithRawResponse:
    def __init__(self, tables: TablesResource) -> None:
        self._tables = tables

        self.list = to_raw_response_wrapper(
            tables.list,
        )

    @cached_property
    def maintenance_configs(self) -> MaintenanceConfigsResourceWithRawResponse:
        return MaintenanceConfigsResourceWithRawResponse(self._tables.maintenance_configs)


class AsyncTablesResourceWithRawResponse:
    def __init__(self, tables: AsyncTablesResource) -> None:
        self._tables = tables

        self.list = async_to_raw_response_wrapper(
            tables.list,
        )

    @cached_property
    def maintenance_configs(self) -> AsyncMaintenanceConfigsResourceWithRawResponse:
        return AsyncMaintenanceConfigsResourceWithRawResponse(self._tables.maintenance_configs)


class TablesResourceWithStreamingResponse:
    def __init__(self, tables: TablesResource) -> None:
        self._tables = tables

        self.list = to_streamed_response_wrapper(
            tables.list,
        )

    @cached_property
    def maintenance_configs(self) -> MaintenanceConfigsResourceWithStreamingResponse:
        return MaintenanceConfigsResourceWithStreamingResponse(self._tables.maintenance_configs)


class AsyncTablesResourceWithStreamingResponse:
    def __init__(self, tables: AsyncTablesResource) -> None:
        self._tables = tables

        self.list = async_to_streamed_response_wrapper(
            tables.list,
        )

    @cached_property
    def maintenance_configs(self) -> AsyncMaintenanceConfigsResourceWithStreamingResponse:
        return AsyncMaintenanceConfigsResourceWithStreamingResponse(self._tables.maintenance_configs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo
from ..buckets.provider import Provider

__all__ = ["ConnectivityPrecheckTargetParams", "Secret"]


class ConnectivityPrecheckTargetParams(TypedDict, total=False):
    account_id: Required[str]

    bucket: Required[str]

    secret: Required[Secret]

    vendor: Required[Provider]

    jurisdiction: Literal["default", "eu", "fedramp"]


class Secret(TypedDict, total=False):
    access_key_id: Required[Annotated[str, PropertyInfo(alias="accessKeyId")]]

    secret_access_key: Required[Annotated[str, PropertyInfo(alias="secretAccessKey")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .hostname import Hostname as Hostname
from .redirect import Redirect as Redirect
from .lists_list import ListsList as ListsList
from .hostname_param import HostnameParam as HostnameParam
from .redirect_param import RedirectParam as RedirectParam
from .list_get_response import ListGetResponse as ListGetResponse
from .list_create_params import ListCreateParams as ListCreateParams
from .list_update_params import ListUpdateParams as ListUpdateParams
from .list_create_response import ListCreateResponse as ListCreateResponse
from .list_delete_response import ListDeleteResponse as ListDeleteResponse
from .list_update_response import ListUpdateResponse as ListUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SiteInfoListParams"]


class SiteInfoListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    order_by: Literal["host", "created"]
    """The property used to sort the list of results."""

    page: float
    """Current page within the paginated list of results."""

    per_page: float
    """Number of items to return per page of results."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BucketCreateParams"]


class BucketCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    name: Required[str]
    """Name of the bucket."""

    location_hint: Annotated[Literal["apac", "eeur", "enam", "weur", "wnam", "oc"], PropertyInfo(alias="locationHint")]
    """Location of the bucket."""

    storage_class: Annotated[Literal["Standard", "InfrequentAccess"], PropertyInfo(alias="storageClass")]
    """Storage class for newly uploaded objects, unless specified otherwise."""

    jurisdiction: Annotated[Literal["default", "eu", "fedramp"], PropertyInfo(alias="cf-r2-jurisdiction")]
    """Jurisdiction where objects in this bucket are guaranteed to be stored."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.leaked_credential_checks import detection_create_params, detection_update_params
from ...types.leaked_credential_checks.detection_get_response import DetectionGetResponse
from ...types.leaked_credential_checks.detection_list_response import DetectionListResponse
from ...types.leaked_credential_checks.detection_create_response import DetectionCreateResponse
from ...types.leaked_credential_checks.detection_update_response import DetectionUpdateResponse

__all__ = ["DetectionsResource", "AsyncDetectionsResource"]


class DetectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DetectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DetectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DetectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DetectionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        password: str | Omit = omit,
        username: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DetectionCreateResponse:
        """
        Create user-defined detection pattern for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          password: Defines ehe ruleset expression to use in matching the password in a request.

          username: Defines the ruleset expression to use in matching the username in a request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/leaked-credential-checks/detections", zone_id=zone_id),
            body=maybe_transform(
                {
                    "password": password,
                    "username": username,
                },
                detection_create_params.DetectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DetectionCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DetectionCreateResponse], ResultWrapper[DetectionCreateResponse]),
        )

    def update(
        self,
        detection_id: str,
        *,
        zone_id: str,
        password: str | Omit = omit,
        username: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DetectionUpdateResponse:
        """
        Update user-defined detection pattern for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          detection_id: Defines the unique ID for this custom detection.

          password: Defines ehe ruleset expression to use in matching the password in a request.

          username: Defines the ruleset expression to use in matching the username in a request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not detection_id:
            raise ValueError(f"Expected a non-empty value for `detection_id` but received {detection_id!r}")
        return self._put(
            path_template(
                "/zones/{zone_id}/leaked-credential-checks/detections/{detection_id}",
                zone_id=zone_id,
                detection_id=detection_id,
            ),
            body=maybe_transform(
                {
                    "password": password,
                    "username": username,
                },
                detection_update_params.DetectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DetectionUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DetectionUpdateResponse], ResultWrapper[DetectionUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DetectionListResponse]:
        """
        List user-defined detection patterns for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/leaked-credential-checks/detections", zone_id=zone_id),
            page=SyncSinglePage[DetectionListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DetectionListResponse,
        )

    def delete(
        self,
        detection_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Remove user-defined detection pattern for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          detection_id: Defines the unique ID for this custom detection.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not detection_id:
            raise ValueError(f"Expected a non-empty value for `detection_id` but received {detection_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/leaked-credential-checks/detections/{detection_id}",
                zone_id=zone_id,
                detection_id=detection_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        detection_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DetectionGetResponse:
        """
        Get user-defined detection pattern for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          detection_id: Defines the unique ID for this custom detection.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not detection_id:
            raise ValueError(f"Expected a non-empty value for `detection_id` but received {detection_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/leaked-credential-checks/detections/{detection_id}",
                zone_id=zone_id,
                detection_id=detection_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DetectionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[DetectionGetResponse], ResultWrapper[DetectionGetResponse]),
        )


class AsyncDetectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDetectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDetectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDetectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDetectionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        password: str | Omit = omit,
        username: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DetectionCreateResponse:
        """
        Create user-defined detection pattern for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          password: Defines ehe ruleset expression to use in matching the password in a request.

          username: Defines the ruleset expression to use in matching the username in a request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/leaked-credential-checks/detections", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "password": password,
                    "username": username,
                },
                detection_create_params.DetectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DetectionCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DetectionCreateResponse], ResultWrapper[DetectionCreateResponse]),
        )

    async def update(
        self,
        detection_id: str,
        *,
        zone_id: str,
        password: str | Omit = omit,
        username: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DetectionUpdateResponse:
        """
        Update user-defined detection pattern for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          detection_id: Defines the unique ID for this custom detection.

          password: Defines ehe ruleset expression to use in matching the password in a request.

          username: Defines the ruleset expression to use in matching the username in a request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not detection_id:
            raise ValueError(f"Expected a non-empty value for `detection_id` but received {detection_id!r}")
        return await self._put(
            path_template(
                "/zones/{zone_id}/leaked-credential-checks/detections/{detection_id}",
                zone_id=zone_id,
                detection_id=detection_id,
            ),
            body=await async_maybe_transform(
                {
                    "password": password,
                    "username": username,
                },
                detection_update_params.DetectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DetectionUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DetectionUpdateResponse], ResultWrapper[DetectionUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DetectionListResponse, AsyncSinglePage[DetectionListResponse]]:
        """
        List user-defined detection patterns for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/leaked-credential-checks/detections", zone_id=zone_id),
            page=AsyncSinglePage[DetectionListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DetectionListResponse,
        )

    async def delete(
        self,
        detection_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Remove user-defined detection pattern for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          detection_id: Defines the unique ID for this custom detection.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not detection_id:
            raise ValueError(f"Expected a non-empty value for `detection_id` but received {detection_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/leaked-credential-checks/detections/{detection_id}",
                zone_id=zone_id,
                detection_id=detection_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        detection_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DetectionGetResponse:
        """
        Get user-defined detection pattern for Leaked Credential Checks.

        Args:
          zone_id: Defines an identifier.

          detection_id: Defines the unique ID for this custom detection.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not detection_id:
            raise ValueError(f"Expected a non-empty value for `detection_id` but received {detection_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/leaked-credential-checks/detections/{detection_id}",
                zone_id=zone_id,
                detection_id=detection_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DetectionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[DetectionGetResponse], ResultWrapper[DetectionGetResponse]),
        )


class DetectionsResourceWithRawResponse:
    def __init__(self, detections: DetectionsResource) -> None:
        self._detections = detections

        self.create = to_raw_response_wrapper(
            detections.create,
        )
        self.update = to_raw_response_wrapper(
            detections.update,
        )
        self.list = to_raw_response_wrapper(
            detections.list,
        )
        self.delete = to_raw_response_wrapper(
            detections.delete,
        )
        self.get = to_raw_response_wrapper(
            detections.get,
        )


class AsyncDetectionsResourceWithRawResponse:
    def __init__(self, detections: AsyncDetectionsResource) -> None:
        self._detections = detections

        self.create = async_to_raw_response_wrapper(
            detections.create,
        )
        self.update = async_to_raw_response_wrapper(
            detections.update,
        )
        self.list = async_to_raw_response_wrapper(
            detections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            detections.delete,
        )
        self.get = async_to_raw_response_wrapper(
            detections.get,
        )


class DetectionsResourceWithStreamingResponse:
    def __init__(self, detections: DetectionsResource) -> None:
        self._detections = detections

        self.create = to_streamed_response_wrapper(
            detections.create,
        )
        self.update = to_streamed_response_wrapper(
            detections.update,
        )
        self.list = to_streamed_response_wrapper(
            detections.list,
        )
        self.delete = to_streamed_response_wrapper(
            detections.delete,
        )
        self.get = to_streamed_response_wrapper(
            detections.get,
        )


class AsyncDetectionsResourceWithStreamingResponse:
    def __init__(self, detections: AsyncDetectionsResource) -> None:
        self._detections = detections

        self.create = async_to_streamed_response_wrapper(
            detections.create,
        )
        self.update = async_to_streamed_response_wrapper(
            detections.update,
        )
        self.list = async_to_streamed_response_wrapper(
            detections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            detections.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            detections.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["IndicatorFeedUpdateParams"]


class IndicatorFeedUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    description: str
    """The new description of the feed"""

    is_attributable: bool
    """The new is_attributable value of the feed"""

    is_downloadable: bool
    """The new is_downloadable value of the feed"""

    is_public: bool
    """The new is_public value of the feed"""

    name: str
    """The new name of the feed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Type"]

Type: TypeAlias = Literal["full", "partial", "secondary", "internal"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["AddressMapEditParams"]


class AddressMapEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier of a Cloudflare account."""

    default_sni: Optional[str]
    """
    If you have legacy TLS clients which do not send the TLS server name indicator,
    then you can specify one default SNI on the map. If Cloudflare receives a TLS
    handshake from a client without an SNI, it will respond with the default SNI on
    those IPs. The default SNI can be any valid zone or subdomain owned by the
    account.
    """

    description: Optional[str]
    """
    An optional description field which may be used to describe the types of IPs or
    zones on the map.
    """

    enabled: Optional[bool]
    """Whether the Address Map is enabled or not.

    Cloudflare's DNS will not respond with IP addresses on an Address Map until the
    map is enabled.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ips import IPs as IPs
from .ip_list_params import IPListParams as IPListParams
from .ip_list_response import IPListResponse as IPListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = [
    "LoggingUpdateParams",
    "SettingsByRuleType",
    "SettingsByRuleTypeDNS",
    "SettingsByRuleTypeHTTP",
    "SettingsByRuleTypeL4",
]


class LoggingUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    redact_pii: bool
    """
    Indicate whether to redact personally identifiable information from activity
    logging (PII fields include source IP, user email, user ID, device ID, URL,
    referrer, and user agent).
    """

    settings_by_rule_type: SettingsByRuleType
    """Configure logging settings for each rule type."""


class SettingsByRuleTypeDNS(TypedDict, total=False):
    """Configure logging settings for DNS firewall."""

    log_all: bool
    """Specify whether to log all requests to this service."""

    log_blocks: bool
    """Specify whether to log only blocking requests to this service."""


class SettingsByRuleTypeHTTP(TypedDict, total=False):
    """Configure logging settings for HTTP/HTTPS firewall."""

    log_all: bool
    """Specify whether to log all requests to this service."""

    log_blocks: bool
    """Specify whether to log only blocking requests to this service."""


class SettingsByRuleTypeL4(TypedDict, total=False):
    """Configure logging settings for Network firewall."""

    log_all: bool
    """Specify whether to log all requests to this service."""

    log_blocks: bool
    """Specify whether to log only blocking requests to this service."""


class SettingsByRuleType(TypedDict, total=False):
    """Configure logging settings for each rule type."""

    dns: SettingsByRuleTypeDNS
    """Configure logging settings for DNS firewall."""

    http: SettingsByRuleTypeHTTP
    """Configure logging settings for HTTP/HTTPS firewall."""

    l4: SettingsByRuleTypeL4
    """Configure logging settings for Network firewall."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .access_request import AccessRequest

__all__ = ["AccessRequestListResponse"]

AccessRequestListResponse: TypeAlias = List[AccessRequest]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.user.tokens import value_update_params
from ....types.shared.token_value import TokenValue

__all__ = ["ValueResource", "AsyncValueResource"]


class ValueResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ValueResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ValueResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ValueResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ValueResourceWithStreamingResponse(self)

    def update(
        self,
        token_id: str,
        *,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Roll the token secret.

        Args:
          token_id: Token identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not token_id:
            raise ValueError(f"Expected a non-empty value for `token_id` but received {token_id!r}")
        return self._put(
            path_template("/user/tokens/{token_id}/value", token_id=token_id),
            body=maybe_transform(body, value_update_params.ValueUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenValue]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class AsyncValueResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncValueResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncValueResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncValueResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncValueResourceWithStreamingResponse(self)

    async def update(
        self,
        token_id: str,
        *,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Roll the token secret.

        Args:
          token_id: Token identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not token_id:
            raise ValueError(f"Expected a non-empty value for `token_id` but received {token_id!r}")
        return await self._put(
            path_template("/user/tokens/{token_id}/value", token_id=token_id),
            body=await async_maybe_transform(body, value_update_params.ValueUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TokenValue]]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class ValueResourceWithRawResponse:
    def __init__(self, value: ValueResource) -> None:
        self._value = value

        self.update = to_raw_response_wrapper(
            value.update,
        )


class AsyncValueResourceWithRawResponse:
    def __init__(self, value: AsyncValueResource) -> None:
        self._value = value

        self.update = async_to_raw_response_wrapper(
            value.update,
        )


class ValueResourceWithStreamingResponse:
    def __init__(self, value: ValueResource) -> None:
        self._value = value

        self.update = to_streamed_response_wrapper(
            value.update,
        )


class AsyncValueResourceWithStreamingResponse:
    def __init__(self, value: AsyncValueResource) -> None:
        self._value = value

        self.update = async_to_streamed_response_wrapper(
            value.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Union, cast
from typing_extensions import Literal

import httpx

from .fields import (
    FieldsResource,
    AsyncFieldsResource,
    FieldsResourceWithRawResponse,
    AsyncFieldsResourceWithRawResponse,
    FieldsResourceWithStreamingResponse,
    AsyncFieldsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....types.logs import received_get_params
from ...._base_client import make_request_options
from ....types.logs.received_get_response import ReceivedGetResponse

__all__ = ["ReceivedResource", "AsyncReceivedResource"]


class ReceivedResource(SyncAPIResource):
    @cached_property
    def fields(self) -> FieldsResource:
        return FieldsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ReceivedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ReceivedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReceivedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ReceivedResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        zone_id: str,
        end: Union[str, int],
        count: int | Omit = omit,
        fields: str | Omit = omit,
        sample: float | Omit = omit,
        start: Union[str, int] | Omit = omit,
        timestamps: Literal["unix", "unixnano", "rfc3339"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReceivedGetResponse:
        """The `/received` api route allows customers to retrieve their edge HTTP logs.

        The
        basic access pattern is "give me all the logs for zone Z for minute M", where
        the minute M refers to the time records were received at Cloudflare's central
        data center. `start` is inclusive, and `end` is exclusive. Because of that, to
        get all data, at minutely cadence, starting at 10AM, the proper values are:
        `start=2018-05-20T10:00:00Z&end=2018-05-20T10:01:00Z`, then
        `start=2018-05-20T10:01:00Z&end=2018-05-20T10:02:00Z` and so on; the overlap
        will be handled properly.

        Args:
          zone_id: Identifier.

          end: Sets the (exclusive) end of the requested time frame. This can be a unix
              timestamp (in seconds or nanoseconds), or an absolute timestamp that conforms to
              RFC 3339. `end` must be at least five minutes earlier than now and must be later
              than `start`. Difference between `start` and `end` must be not greater than one
              hour.

          count: When `?count=` is provided, the response will contain up to `count` results.
              Since results are not sorted, you are likely to get different data for repeated
              requests. `count` must be an integer > 0.

          fields: The `/received` route by default returns a limited set of fields, and allows
              customers to override the default field set by specifying individual fields. The
              reasons for this are: 1. Most customers require only a small subset of fields,
              but that subset varies from customer to customer; 2. Flat schema is much easier
              to work with downstream (importing into BigTable etc); 3. Performance (time to
              process, file size). If `?fields=` is not specified, default field set is
              returned. This default field set may change at any time. When `?fields=` is
              provided, each record is returned with the specified fields. `fields` must be
              specified as a comma separated list without any whitespaces, and all fields must
              exist. The order in which fields are specified does not matter, and the order of
              fields in the response is not specified.

          sample: When `?sample=` is provided, a sample of matching records is returned. If
              `sample=0.1` then 10% of records will be returned. Sampling is random: repeated
              calls will not only return different records, but likely will also vary slightly
              in number of returned records. When `?count=` is also specified, `count` is
              applied to the number of returned records, not the sampled records. So, with
              `sample=0.05` and `count=7`, when there is a total of 100 records available,
              approximately five will be returned. When there are 1000 records, seven will be
              returned. When there are 10,000 records, seven will be returned.

          start: Sets the (inclusive) beginning of the requested time frame. This can be a unix
              timestamp (in seconds or nanoseconds), or an absolute timestamp that conforms to
              RFC 3339. At this point in time, it cannot exceed a time in the past greater
              than seven days.

          timestamps: By default, timestamps in responses are returned as Unix nanosecond integers.
              The `?timestamps=` argument can be set to change the format in which response
              timestamps are returned. Possible values are: `unix`, `unixnano`, `rfc3339`.
              Note that `unix` and `unixnano` return timestamps as integers; `rfc3339` returns
              timestamps as strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            ReceivedGetResponse,
            self._get(
                path_template("/zones/{zone_id}/logs/received", zone_id=zone_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform(
                        {
                            "end": end,
                            "count": count,
                            "fields": fields,
                            "sample": sample,
                            "start": start,
                            "timestamps": timestamps,
                        },
                        received_get_params.ReceivedGetParams,
                    ),
                ),
                cast_to=cast(
                    Any, ReceivedGetResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncReceivedResource(AsyncAPIResource):
    @cached_property
    def fields(self) -> AsyncFieldsResource:
        return AsyncFieldsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncReceivedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReceivedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReceivedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncReceivedResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        zone_id: str,
        end: Union[str, int],
        count: int | Omit = omit,
        fields: str | Omit = omit,
        sample: float | Omit = omit,
        start: Union[str, int] | Omit = omit,
        timestamps: Literal["unix", "unixnano", "rfc3339"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ReceivedGetResponse:
        """The `/received` api route allows customers to retrieve their edge HTTP logs.

        The
        basic access pattern is "give me all the logs for zone Z for minute M", where
        the minute M refers to the time records were received at Cloudflare's central
        data center. `start` is inclusive, and `end` is exclusive. Because of that, to
        get all data, at minutely cadence, starting at 10AM, the proper values are:
        `start=2018-05-20T10:00:00Z&end=2018-05-20T10:01:00Z`, then
        `start=2018-05-20T10:01:00Z&end=2018-05-20T10:02:00Z` and so on; the overlap
        will be handled properly.

        Args:
          zone_id: Identifier.

          end: Sets the (exclusive) end of the requested time frame. This can be a unix
              timestamp (in seconds or nanoseconds), or an absolute timestamp that conforms to
              RFC 3339. `end` must be at least five minutes earlier than now and must be later
              than `start`. Difference between `start` and `end` must be not greater than one
              hour.

          count: When `?count=` is provided, the response will contain up to `count` results.
              Since results are not sorted, you are likely to get different data for repeated
              requests. `count` must be an integer > 0.

          fields: The `/received` route by default returns a limited set of fields, and allows
              customers to override the default field set by specifying individual fields. The
              reasons for this are: 1. Most customers require only a small subset of fields,
              but that subset varies from customer to customer; 2. Flat schema is much easier
              to work with downstream (importing into BigTable etc); 3. Performance (time to
              process, file size). If `?fields=` is not specified, default field set is
              returned. This default field set may change at any time. When `?fields=` is
              provided, each record is returned with the specified fields. `fields` must be
              specified as a comma separated list without any whitespaces, and all fields must
              exist. The order in which fields are specified does not matter, and the order of
              fields in the response is not specified.

          sample: When `?sample=` is provided, a sample of matching records is returned. If
              `sample=0.1` then 10% of records will be returned. Sampling is random: repeated
              calls will not only return different records, but likely will also vary slightly
              in number of returned records. When `?count=` is also specified, `count` is
              applied to the number of returned records, not the sampled records. So, with
              `sample=0.05` and `count=7`, when there is a total of 100 records available,
              approximately five will be returned. When there are 1000 records, seven will be
              returned. When there are 10,000 records, seven will be returned.

          start: Sets the (inclusive) beginning of the requested time frame. This can be a unix
              timestamp (in seconds or nanoseconds), or an absolute timestamp that conforms to
              RFC 3339. At this point in time, it cannot exceed a time in the past greater
              than seven days.

          timestamps: By default, timestamps in responses are returned as Unix nanosecond integers.
              The `?timestamps=` argument can be set to change the format in which response
              timestamps are returned. Possible values are: `unix`, `unixnano`, `rfc3339`.
              Note that `unix` and `unixnano` return timestamps as integers; `rfc3339` returns
              timestamps as strings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            ReceivedGetResponse,
            await self._get(
                path_template("/zones/{zone_id}/logs/received", zone_id=zone_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform(
                        {
                            "end": end,
                            "count": count,
                            "fields": fields,
                            "sample": sample,
                            "start": start,
                            "timestamps": timestamps,
                        },
                        received_get_params.ReceivedGetParams,
                    ),
                ),
                cast_to=cast(
                    Any, ReceivedGetResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class ReceivedResourceWithRawResponse:
    def __init__(self, received: ReceivedResource) -> None:
        self._received = received

        self.get = to_raw_response_wrapper(
            received.get,
        )

    @cached_property
    def fields(self) -> FieldsResourceWithRawResponse:
        return FieldsResourceWithRawResponse(self._received.fields)


class AsyncReceivedResourceWithRawResponse:
    def __init__(self, received: AsyncReceivedResource) -> None:
        self._received = received

        self.get = async_to_raw_response_wrapper(
            received.get,
        )

    @cached_property
    def fields(self) -> AsyncFieldsResourceWithRawResponse:
        return AsyncFieldsResourceWithRawResponse(self._received.fields)


class ReceivedResourceWithStreamingResponse:
    def __init__(self, received: ReceivedResource) -> None:
        self._received = received

        self.get = to_streamed_response_wrapper(
            received.get,
        )

    @cached_property
    def fields(self) -> FieldsResourceWithStreamingResponse:
        return FieldsResourceWithStreamingResponse(self._received.fields)


class AsyncReceivedResourceWithStreamingResponse:
    def __init__(self, received: AsyncReceivedResource) -> None:
        self._received = received

        self.get = async_to_streamed_response_wrapper(
            received.get,
        )

    @cached_property
    def fields(self) -> AsyncFieldsResourceWithStreamingResponse:
        return AsyncFieldsResourceWithStreamingResponse(self._received.fields)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["HoldCreateParams"]


class HoldCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    include_subdomains: bool
    """
    If provided, the zone hold will extend to block any subdomain of the given zone,
    as well as SSL4SaaS Custom Hostnames. For example, a zone hold on a zone with
    the hostname 'example.com' and include_subdomains=true will block 'example.com',
    'staging.example.com', 'api.staging.example.com', etc.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.email_security.investigate import move_bulk_params, move_create_params
from ....types.email_security.investigate.move_bulk_response import MoveBulkResponse
from ....types.email_security.investigate.move_create_response import MoveCreateResponse

__all__ = ["MoveResource", "AsyncMoveResource"]


class MoveResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MoveResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MoveResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MoveResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MoveResourceWithStreamingResponse(self)

    def create(
        self,
        investigate_id: str,
        *,
        account_id: str,
        destination: Literal[
            "Inbox", "JunkEmail", "DeletedItems", "RecoverableItemsDeletions", "RecoverableItemsPurges"
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[MoveCreateResponse]:
        """
        Moves a single message to a specified mailbox folder (Inbox, JunkEmail,
        DeletedItems, RecoverableItemsDeletions, or RecoverableItemsPurges). Requires
        active integration.

        Args:
          account_id: Identifier.

          investigate_id: Unique identifier for a message retrieved from investigation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not investigate_id:
            raise ValueError(f"Expected a non-empty value for `investigate_id` but received {investigate_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/email-security/investigate/{investigate_id}/move",
                account_id=account_id,
                investigate_id=investigate_id,
            ),
            page=SyncSinglePage[MoveCreateResponse],
            body=maybe_transform({"destination": destination}, move_create_params.MoveCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=MoveCreateResponse,
            method="post",
        )

    def bulk(
        self,
        *,
        account_id: str,
        destination: Literal[
            "Inbox", "JunkEmail", "DeletedItems", "RecoverableItemsDeletions", "RecoverableItemsPurges"
        ],
        ids: SequenceNotStr[str] | Omit = omit,
        postfix_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[MoveBulkResponse]:
        """
        Moves multiple messages to a specified mailbox folder (Inbox, JunkEmail,
        DeletedItems, RecoverableItemsDeletions, or RecoverableItemsPurges). Requires
        active integration.

        Args:
          account_id: Identifier.

          ids: List of message IDs to move

          postfix_ids: Deprecated, use `ids` instead. End of life: November 1, 2026. List of message
              IDs to move.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/investigate/move", account_id=account_id),
            page=SyncSinglePage[MoveBulkResponse],
            body=maybe_transform(
                {
                    "destination": destination,
                    "ids": ids,
                    "postfix_ids": postfix_ids,
                },
                move_bulk_params.MoveBulkParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=MoveBulkResponse,
            method="post",
        )


class AsyncMoveResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMoveResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMoveResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMoveResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMoveResourceWithStreamingResponse(self)

    def create(
        self,
        investigate_id: str,
        *,
        account_id: str,
        destination: Literal[
            "Inbox", "JunkEmail", "DeletedItems", "RecoverableItemsDeletions", "RecoverableItemsPurges"
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MoveCreateResponse, AsyncSinglePage[MoveCreateResponse]]:
        """
        Moves a single message to a specified mailbox folder (Inbox, JunkEmail,
        DeletedItems, RecoverableItemsDeletions, or RecoverableItemsPurges). Requires
        active integration.

        Args:
          account_id: Identifier.

          investigate_id: Unique identifier for a message retrieved from investigation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not investigate_id:
            raise ValueError(f"Expected a non-empty value for `investigate_id` but received {investigate_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/email-security/investigate/{investigate_id}/move",
                account_id=account_id,
                investigate_id=investigate_id,
            ),
            page=AsyncSinglePage[MoveCreateResponse],
            body=maybe_transform({"destination": destination}, move_create_params.MoveCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=MoveCreateResponse,
            method="post",
        )

    def bulk(
        self,
        *,
        account_id: str,
        destination: Literal[
            "Inbox", "JunkEmail", "DeletedItems", "RecoverableItemsDeletions", "RecoverableItemsPurges"
        ],
        ids: SequenceNotStr[str] | Omit = omit,
        postfix_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MoveBulkResponse, AsyncSinglePage[MoveBulkResponse]]:
        """
        Moves multiple messages to a specified mailbox folder (Inbox, JunkEmail,
        DeletedItems, RecoverableItemsDeletions, or RecoverableItemsPurges). Requires
        active integration.

        Args:
          account_id: Identifier.

          ids: List of message IDs to move

          postfix_ids: Deprecated, use `ids` instead. End of life: November 1, 2026. List of message
              IDs to move.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/investigate/move", account_id=account_id),
            page=AsyncSinglePage[MoveBulkResponse],
            body=maybe_transform(
                {
                    "destination": destination,
                    "ids": ids,
                    "postfix_ids": postfix_ids,
                },
                move_bulk_params.MoveBulkParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=MoveBulkResponse,
            method="post",
        )


class MoveResourceWithRawResponse:
    def __init__(self, move: MoveResource) -> None:
        self._move = move

        self.create = to_raw_response_wrapper(
            move.create,
        )
        self.bulk = to_raw_response_wrapper(
            move.bulk,
        )


class AsyncMoveResourceWithRawResponse:
    def __init__(self, move: AsyncMoveResource) -> None:
        self._move = move

        self.create = async_to_raw_response_wrapper(
            move.create,
        )
        self.bulk = async_to_raw_response_wrapper(
            move.bulk,
        )


class MoveResourceWithStreamingResponse:
    def __init__(self, move: MoveResource) -> None:
        self._move = move

        self.create = to_streamed_response_wrapper(
            move.create,
        )
        self.bulk = to_streamed_response_wrapper(
            move.bulk,
        )


class AsyncMoveResourceWithStreamingResponse:
    def __init__(self, move: AsyncMoveResource) -> None:
        self._move = move

        self.create = async_to_streamed_response_wrapper(
            move.create,
        )
        self.bulk = async_to_streamed_response_wrapper(
            move.bulk,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "CloudIntegrationEditResponse",
    "Status",
    "StatusDiscoveryProgress",
    "StatusDiscoveryProgressV2",
    "StatusInUseBy",
]


class StatusDiscoveryProgress(BaseModel):
    done: int

    total: int

    unit: str


class StatusDiscoveryProgressV2(BaseModel):
    done: int

    total: int

    unit: str


class StatusInUseBy(BaseModel):
    id: str

    client_type: Literal["MAGIC_WAN_CLOUD_ONRAMP"]

    name: str


class Status(BaseModel):
    discovery_progress: StatusDiscoveryProgress

    discovery_progress_v2: StatusDiscoveryProgressV2

    last_discovery_status: Literal["UNSPECIFIED", "PENDING", "DISCOVERING", "FAILED", "SUCCEEDED"]

    last_discovery_status_v2: Literal["UNSPECIFIED", "PENDING", "DISCOVERING", "FAILED", "SUCCEEDED"]

    regions: List[str]

    credentials_good_since: Optional[str] = None

    credentials_missing_since: Optional[str] = None

    credentials_rejected_since: Optional[str] = None

    discovery_message: Optional[str] = None

    discovery_message_v2: Optional[str] = None

    in_use_by: Optional[List[StatusInUseBy]] = None

    last_discovery_completed_at: Optional[str] = None

    last_discovery_completed_at_v2: Optional[str] = None

    last_discovery_started_at: Optional[str] = None

    last_discovery_started_at_v2: Optional[str] = None

    last_updated: Optional[str] = None


class CloudIntegrationEditResponse(BaseModel):
    id: str

    cloud_type: Literal["AWS", "AZURE", "GOOGLE", "CLOUDFLARE"]

    friendly_name: str

    last_updated: str

    lifecycle_state: Literal["ACTIVE", "PENDING_SETUP", "RETIRED"]

    state: Literal["UNSPECIFIED", "PENDING", "DISCOVERING", "FAILED", "SUCCEEDED"]

    state_v2: Literal["UNSPECIFIED", "PENDING", "DISCOVERING", "FAILED", "SUCCEEDED"]

    aws_arn: Optional[str] = None

    azure_subscription_id: Optional[str] = None

    azure_tenant_id: Optional[str] = None

    description: Optional[str] = None

    gcp_project_id: Optional[str] = None

    gcp_service_account_email: Optional[str] = None

    status: Optional[Status] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.ai import markdown_for_agent_summary_params, markdown_for_agent_timeseries_params
from ....types.radar.ai.markdown_for_agent_summary_response import MarkdownForAgentSummaryResponse
from ....types.radar.ai.markdown_for_agent_timeseries_response import MarkdownForAgentTimeseriesResponse

__all__ = ["MarkdownForAgentsResource", "AsyncMarkdownForAgentsResource"]


class MarkdownForAgentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MarkdownForAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MarkdownForAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MarkdownForAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MarkdownForAgentsResourceWithStreamingResponse(self)

    def summary(
        self,
        *,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MarkdownForAgentSummaryResponse:
        """
        Retrieves the overall median HTML-to-markdown reduction ratio for AI agent
        requests over the given date range.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/ai/markdown_for_agents/summary",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "name": name,
                    },
                    markdown_for_agent_summary_params.MarkdownForAgentSummaryParams,
                ),
                post_parser=ResultWrapper[MarkdownForAgentSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[MarkdownForAgentSummaryResponse], ResultWrapper[MarkdownForAgentSummaryResponse]),
        )

    def timeseries(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MarkdownForAgentTimeseriesResponse:
        """
        Retrieves the median HTML-to-markdown reduction ratio over time for AI agent
        requests.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/ai/markdown_for_agents/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "name": name,
                    },
                    markdown_for_agent_timeseries_params.MarkdownForAgentTimeseriesParams,
                ),
                post_parser=ResultWrapper[MarkdownForAgentTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[MarkdownForAgentTimeseriesResponse], ResultWrapper[MarkdownForAgentTimeseriesResponse]),
        )


class AsyncMarkdownForAgentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMarkdownForAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMarkdownForAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMarkdownForAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMarkdownForAgentsResourceWithStreamingResponse(self)

    async def summary(
        self,
        *,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MarkdownForAgentSummaryResponse:
        """
        Retrieves the overall median HTML-to-markdown reduction ratio for AI agent
        requests over the given date range.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/ai/markdown_for_agents/summary",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "name": name,
                    },
                    markdown_for_agent_summary_params.MarkdownForAgentSummaryParams,
                ),
                post_parser=ResultWrapper[MarkdownForAgentSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[MarkdownForAgentSummaryResponse], ResultWrapper[MarkdownForAgentSummaryResponse]),
        )

    async def timeseries(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MarkdownForAgentTimeseriesResponse:
        """
        Retrieves the median HTML-to-markdown reduction ratio over time for AI agent
        requests.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/ai/markdown_for_agents/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "name": name,
                    },
                    markdown_for_agent_timeseries_params.MarkdownForAgentTimeseriesParams,
                ),
                post_parser=ResultWrapper[MarkdownForAgentTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[MarkdownForAgentTimeseriesResponse], ResultWrapper[MarkdownForAgentTimeseriesResponse]),
        )


class MarkdownForAgentsResourceWithRawResponse:
    def __init__(self, markdown_for_agents: MarkdownForAgentsResource) -> None:
        self._markdown_for_agents = markdown_for_agents

        self.summary = to_raw_response_wrapper(
            markdown_for_agents.summary,
        )
        self.timeseries = to_raw_response_wrapper(
            markdown_for_agents.timeseries,
        )


class AsyncMarkdownForAgentsResourceWithRawResponse:
    def __init__(self, markdown_for_agents: AsyncMarkdownForAgentsResource) -> None:
        self._markdown_for_agents = markdown_for_agents

        self.summary = async_to_raw_response_wrapper(
            markdown_for_agents.summary,
        )
        self.timeseries = async_to_raw_response_wrapper(
            markdown_for_agents.timeseries,
        )


class MarkdownForAgentsResourceWithStreamingResponse:
    def __init__(self, markdown_for_agents: MarkdownForAgentsResource) -> None:
        self._markdown_for_agents = markdown_for_agents

        self.summary = to_streamed_response_wrapper(
            markdown_for_agents.summary,
        )
        self.timeseries = to_streamed_response_wrapper(
            markdown_for_agents.timeseries,
        )


class AsyncMarkdownForAgentsResourceWithStreamingResponse:
    def __init__(self, markdown_for_agents: AsyncMarkdownForAgentsResource) -> None:
        self._markdown_for_agents = markdown_for_agents

        self.summary = async_to_streamed_response_wrapper(
            markdown_for_agents.summary,
        )
        self.timeseries = async_to_streamed_response_wrapper(
            markdown_for_agents.timeseries,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ....._types import SequenceNotStr

__all__ = ["ViewCreateParams"]


class ViewCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    name: Required[str]
    """The name of the view."""

    zones: Required[SequenceNotStr[str]]
    """The list of zones linked to this view."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["Config"]


class Config(BaseModel):
    """Google Tag Gateway configuration for a zone."""

    enabled: bool
    """Enables or disables Google Tag Gateway for this zone."""

    endpoint: str
    """Specifies the endpoint path for proxying Google Tag Manager requests.

    Use an absolute path starting with '/', with no nested paths and alphanumeric
    characters only (e.g. /metrics).
    """

    hide_original_ip: bool = FieldInfo(alias="hideOriginalIp")
    """Hides the original client IP address from Google when enabled."""

    measurement_id: str = FieldInfo(alias="measurementId")
    """Specify the Google Tag Manager container or measurement ID (e.g.

    GTM-XXXXXXX or G-XXXXXXXXXX).
    """

    set_up_tag: Optional[bool] = FieldInfo(alias="setUpTag", default=None)
    """Set up the associated Google Tag on the zone automatically when enabled."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ServiceListParams"]


class ServiceListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier"""

    page: int
    """Current page in the response"""

    per_page: int
    """Max amount of entries returned per page"""

    type: Optional[Literal["tcp", "http"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["ScriptSearchResponse", "ScriptSearchResponseItem"]


class ScriptSearchResponseItem(BaseModel):
    id: str
    """Identifier."""

    created_on: datetime
    """When the script was created."""

    modified_on: datetime
    """When the script was last modified."""

    script_name: str
    """Name of the script, used in URLs and route configuration."""

    environment_is_default: Optional[bool] = None
    """Whether the environment is the default environment."""

    environment_name: Optional[str] = None
    """Name of the environment."""

    service_name: Optional[str] = None
    """Name of the service."""


ScriptSearchResponse: TypeAlias = List[ScriptSearchResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BotListParams"]


class BotListParams(TypedDict, total=False):
    bot_category: Annotated[
        Literal[
            "SEARCH_ENGINE_CRAWLER",
            "SEARCH_ENGINE_OPTIMIZATION",
            "MONITORING_AND_ANALYTICS",
            "ADVERTISING_AND_MARKETING",
            "SOCIAL_MEDIA_MARKETING",
            "PAGE_PREVIEW",
            "ACADEMIC_RESEARCH",
            "SECURITY",
            "ACCESSIBILITY",
            "WEBHOOKS",
            "FEED_FETCHER",
            "AI_CRAWLER",
            "AGGREGATOR",
            "AI_ASSISTANT",
            "AI_SEARCH",
            "ARCHIVER",
        ],
        PropertyInfo(alias="botCategory"),
    ]
    """Filters results by bot category."""

    bot_operator: Annotated[str, PropertyInfo(alias="botOperator")]
    """Filters results by bot operator."""

    bot_verification_status: Annotated[Literal["VERIFIED"], PropertyInfo(alias="botVerificationStatus")]
    """Filters results by bot verification status."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    kind: Literal["AGENT", "BOT"]
    """Filters results by bot kind."""

    limit: int
    """Limits the number of objects returned in the response."""

    offset: int
    """Skips the specified number of objects before fetching the results."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.radar.bgp.rpki import roa_timeseries_params
from .....types.radar.bgp.rpki.roa_timeseries_response import RoaTimeseriesResponse

__all__ = ["RoasResource", "AsyncRoasResource"]


class RoasResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RoasResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RoasResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RoasResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RoasResourceWithStreamingResponse(self)

    def timeseries(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        metric: Literal[
            "validPfxsRatio",
            "validPfxsV4Ratio",
            "validPfxsV6Ratio",
            "validIpsRatio",
            "validIpsV4Ratio",
            "validIpsV6Ratio",
        ]
        | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoaTimeseriesResponse:
        """
        Retrieves RPKI ROA (Route Origin Authorization) validation ratios over time.
        Returns the selected metric as a time series. Supports filtering by ASN or
        location (country code) — multiple values of the same filter type produce one
        series per value. If no ASN or location is specified, returns the global
        aggregate.

        Args:
          asn: Filters results by Autonomous System Number. Specify one or more ASNs. Multiple
              values generate one series per ASN.

          date_end: End of the date range (inclusive).

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 location
              codes.

          metric: Which RPKI ROA validation metric to return. validPfxsRatio = ratio of RPKI-valid
              prefixes (IPv4+IPv6 combined). validPfxsV4Ratio / validPfxsV6Ratio = same, split
              by IP version. validIpsRatio = ratio of RPKI-valid address space (IPv4 /24s +
              IPv6 /48s). validIpsV4Ratio / validIpsV6Ratio = same, split by IP version.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bgp/rpki/roas/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_start": date_start,
                        "format": format,
                        "location": location,
                        "metric": metric,
                        "name": name,
                    },
                    roa_timeseries_params.RoaTimeseriesParams,
                ),
                post_parser=ResultWrapper[RoaTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[RoaTimeseriesResponse], ResultWrapper[RoaTimeseriesResponse]),
        )


class AsyncRoasResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRoasResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRoasResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRoasResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRoasResourceWithStreamingResponse(self)

    async def timeseries(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        metric: Literal[
            "validPfxsRatio",
            "validPfxsV4Ratio",
            "validPfxsV6Ratio",
            "validIpsRatio",
            "validIpsV4Ratio",
            "validIpsV6Ratio",
        ]
        | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoaTimeseriesResponse:
        """
        Retrieves RPKI ROA (Route Origin Authorization) validation ratios over time.
        Returns the selected metric as a time series. Supports filtering by ASN or
        location (country code) — multiple values of the same filter type produce one
        series per value. If no ASN or location is specified, returns the global
        aggregate.

        Args:
          asn: Filters results by Autonomous System Number. Specify one or more ASNs. Multiple
              values generate one series per ASN.

          date_end: End of the date range (inclusive).

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 location
              codes.

          metric: Which RPKI ROA validation metric to return. validPfxsRatio = ratio of RPKI-valid
              prefixes (IPv4+IPv6 combined). validPfxsV4Ratio / validPfxsV6Ratio = same, split
              by IP version. validIpsRatio = ratio of RPKI-valid address space (IPv4 /24s +
              IPv6 /48s). validIpsV4Ratio / validIpsV6Ratio = same, split by IP version.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bgp/rpki/roas/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_start": date_start,
                        "format": format,
                        "location": location,
                        "metric": metric,
                        "name": name,
                    },
                    roa_timeseries_params.RoaTimeseriesParams,
                ),
                post_parser=ResultWrapper[RoaTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[RoaTimeseriesResponse], ResultWrapper[RoaTimeseriesResponse]),
        )


class RoasResourceWithRawResponse:
    def __init__(self, roas: RoasResource) -> None:
        self._roas = roas

        self.timeseries = to_raw_response_wrapper(
            roas.timeseries,
        )


class AsyncRoasResourceWithRawResponse:
    def __init__(self, roas: AsyncRoasResource) -> None:
        self._roas = roas

        self.timeseries = async_to_raw_response_wrapper(
            roas.timeseries,
        )


class RoasResourceWithStreamingResponse:
    def __init__(self, roas: RoasResource) -> None:
        self._roas = roas

        self.timeseries = to_streamed_response_wrapper(
            roas.timeseries,
        )


class AsyncRoasResourceWithStreamingResponse:
    def __init__(self, roas: AsyncRoasResource) -> None:
        self._roas = roas

        self.timeseries = async_to_streamed_response_wrapper(
            roas.timeseries,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .log_edit_params import LogEditParams as LogEditParams
from .log_list_params import LogListParams as LogListParams
from .log_get_response import LogGetResponse as LogGetResponse
from .url_get_response import URLGetResponse as URLGetResponse
from .log_delete_params import LogDeleteParams as LogDeleteParams
from .log_list_response import LogListResponse as LogListResponse
from .dataset_list_params import DatasetListParams as DatasetListParams
from .log_delete_response import LogDeleteResponse as LogDeleteResponse
from .dataset_get_response import DatasetGetResponse as DatasetGetResponse
from .dataset_create_params import DatasetCreateParams as DatasetCreateParams
from .dataset_list_response import DatasetListResponse as DatasetListResponse
from .dataset_update_params import DatasetUpdateParams as DatasetUpdateParams
from .ai_gateway_list_params import AIGatewayListParams as AIGatewayListParams
from .evaluation_list_params import EvaluationListParams as EvaluationListParams
from .ai_gateway_get_response import AIGatewayGetResponse as AIGatewayGetResponse
from .dataset_create_response import DatasetCreateResponse as DatasetCreateResponse
from .dataset_delete_response import DatasetDeleteResponse as DatasetDeleteResponse
from .dataset_update_response import DatasetUpdateResponse as DatasetUpdateResponse
from .evaluation_get_response import EvaluationGetResponse as EvaluationGetResponse
from .ai_gateway_create_params import AIGatewayCreateParams as AIGatewayCreateParams
from .ai_gateway_list_response import AIGatewayListResponse as AIGatewayListResponse
from .ai_gateway_update_params import AIGatewayUpdateParams as AIGatewayUpdateParams
from .evaluation_create_params import EvaluationCreateParams as EvaluationCreateParams
from .evaluation_list_response import EvaluationListResponse as EvaluationListResponse
from .ai_gateway_create_response import AIGatewayCreateResponse as AIGatewayCreateResponse
from .ai_gateway_delete_response import AIGatewayDeleteResponse as AIGatewayDeleteResponse
from .ai_gateway_update_response import AIGatewayUpdateResponse as AIGatewayUpdateResponse
from .evaluation_create_response import EvaluationCreateResponse as EvaluationCreateResponse
from .evaluation_delete_response import EvaluationDeleteResponse as EvaluationDeleteResponse
from .dynamic_routing_list_params import DynamicRoutingListParams as DynamicRoutingListParams
from .evaluation_type_list_params import EvaluationTypeListParams as EvaluationTypeListParams
from .provider_config_list_params import ProviderConfigListParams as ProviderConfigListParams
from .billing_usage_history_params import BillingUsageHistoryParams as BillingUsageHistoryParams
from .dynamic_routing_get_response import DynamicRoutingGetResponse as DynamicRoutingGetResponse
from .dynamic_routing_create_params import DynamicRoutingCreateParams as DynamicRoutingCreateParams
from .dynamic_routing_list_response import DynamicRoutingListResponse as DynamicRoutingListResponse
from .dynamic_routing_update_params import DynamicRoutingUpdateParams as DynamicRoutingUpdateParams
from .evaluation_type_list_response import EvaluationTypeListResponse as EvaluationTypeListResponse
from .provider_config_create_params import ProviderConfigCreateParams as ProviderConfigCreateParams
from .provider_config_list_response import ProviderConfigListResponse as ProviderConfigListResponse
from .billing_invoice_history_params import BillingInvoiceHistoryParams as BillingInvoiceHistoryParams
from .billing_usage_history_response import BillingUsageHistoryResponse as BillingUsageHistoryResponse
from .billing_credit_balance_response import BillingCreditBalanceResponse as BillingCreditBalanceResponse
from .dynamic_routing_create_response import DynamicRoutingCreateResponse as DynamicRoutingCreateResponse
from .dynamic_routing_delete_response import DynamicRoutingDeleteResponse as DynamicRoutingDeleteResponse
from .dynamic_routing_update_response import DynamicRoutingUpdateResponse as DynamicRoutingUpdateResponse
from .provider_config_create_response import ProviderConfigCreateResponse as ProviderConfigCreateResponse
from .billing_invoice_history_response import BillingInvoiceHistoryResponse as BillingInvoiceHistoryResponse
from .billing_invoice_preview_response import BillingInvoicePreviewResponse as BillingInvoicePreviewResponse
from .dynamic_routing_get_version_response import DynamicRoutingGetVersionResponse as DynamicRoutingGetVersionResponse
from .dynamic_routing_create_version_params import (
    DynamicRoutingCreateVersionParams as DynamicRoutingCreateVersionParams,
)
from .dynamic_routing_list_versions_response import (
    DynamicRoutingListVersionsResponse as DynamicRoutingListVersionsResponse,
)
from .dynamic_routing_create_version_response import (
    DynamicRoutingCreateVersionResponse as DynamicRoutingCreateVersionResponse,
)
from .dynamic_routing_create_deployment_params import (
    DynamicRoutingCreateDeploymentParams as DynamicRoutingCreateDeploymentParams,
)
from .dynamic_routing_list_deployments_response import (
    DynamicRoutingListDeploymentsResponse as DynamicRoutingListDeploymentsResponse,
)
from .dynamic_routing_create_deployment_response import (
    DynamicRoutingCreateDeploymentResponse as DynamicRoutingCreateDeploymentResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, TypedDict

from .output_options_param import OutputOptionsParam

__all__ = ["JobUpdateParams"]


class JobUpdateParams(TypedDict, total=False):
    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    destination_conf: str
    """Uniquely identifies a resource (such as an s3 bucket) where data.

    will be pushed. Additional configuration parameters supported by the destination
    may be included.
    """

    enabled: bool
    """Flag that indicates if the job is enabled."""

    filter: Optional[str]
    """The filters to select the events to include and/or remove from your logs.

    For more information, refer to
    [Filters](https://developers.cloudflare.com/logs/reference/filters/).
    """

    frequency: Optional[Literal["high", "low"]]
    """This field is deprecated.

    Please use `max_upload_*` parameters instead. . The frequency at which
    Cloudflare sends batches of logs to your destination. Setting frequency to high
    sends your logs in larger quantities of smaller files. Setting frequency to low
    sends logs in smaller quantities of larger files.
    """

    kind: Literal["", "edge"]
    """
    The kind parameter (optional) is used to differentiate between Logpush and Edge
    Log Delivery jobs (when supported by the dataset).
    """

    logpull_options: Optional[str]
    """This field is deprecated.

    Use `output_options` instead. Configuration string. It specifies things like
    requested fields and timestamp formats. If migrating from the logpull api, copy
    the url (full url or just the query string) of your call here, and logpush will
    keep on making this call for you, setting start and end times appropriately.
    """

    max_upload_bytes: Union[Literal[0], int, None]
    """The maximum uncompressed file size of a batch of logs.

    This setting value must be between `5 MB` and `1 GB`, or `0` to disable it. Note
    that you cannot set a minimum file size; this means that log files may be much
    smaller than this batch size.
    """

    max_upload_interval_seconds: Union[Literal[0], int, None]
    """The maximum interval in seconds for log batches.

    This setting must be between 30 and 300 seconds (5 minutes), or `0` to disable
    it. Note that you cannot specify a minimum interval for log batches; this means
    that log files may be sent in shorter intervals than this.
    """

    max_upload_records: Union[Literal[0], int, None]
    """The maximum number of log lines per batch.

    This setting must be between 1000 and 1,000,000 lines, or `0` to disable it.
    Note that you cannot specify a minimum number of log lines per batch; this means
    that log files may contain many fewer lines than this.
    """

    name: Optional[str]
    """Optional human readable job name.

    Not unique. Cloudflare suggests. that you set this to a meaningful string, like
    the domain name, to make it easier to identify your job.
    """

    output_options: Optional[OutputOptionsParam]
    """The structured replacement for `logpull_options`.

    When including this field, the `logpull_option` field will be ignored.
    """

    ownership_challenge: str
    """Ownership challenge token to prove destination ownership."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from ......_types import Body, Query, Headers, NotGiven, not_given
from ......_utils import path_template
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......pagination import SyncSinglePage, AsyncSinglePage
from ......_base_client import AsyncPaginator, make_request_options
from ......types.workers_for_platforms.dispatch.namespaces.scripts.binding_get_response import BindingGetResponse

__all__ = ["BindingsResource", "AsyncBindingsResource"]


class BindingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BindingsResourceWithStreamingResponse(self)

    def get(
        self,
        script_name: str,
        *,
        account_id: str,
        dispatch_namespace: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[BindingGetResponse]:
        """
        Fetch script bindings from a script uploaded to a Workers for Platforms
        namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}/scripts/{script_name}/bindings",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
                script_name=script_name,
            ),
            page=SyncSinglePage[BindingGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=cast(Any, BindingGetResponse),  # Union types cannot be passed in as arguments in the type system
        )


class AsyncBindingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBindingsResourceWithStreamingResponse(self)

    def get(
        self,
        script_name: str,
        *,
        account_id: str,
        dispatch_namespace: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BindingGetResponse, AsyncSinglePage[BindingGetResponse]]:
        """
        Fetch script bindings from a script uploaded to a Workers for Platforms
        namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}/scripts/{script_name}/bindings",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
                script_name=script_name,
            ),
            page=AsyncSinglePage[BindingGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=cast(Any, BindingGetResponse),  # Union types cannot be passed in as arguments in the type system
        )


class BindingsResourceWithRawResponse:
    def __init__(self, bindings: BindingsResource) -> None:
        self._bindings = bindings

        self.get = to_raw_response_wrapper(
            bindings.get,
        )


class AsyncBindingsResourceWithRawResponse:
    def __init__(self, bindings: AsyncBindingsResource) -> None:
        self._bindings = bindings

        self.get = async_to_raw_response_wrapper(
            bindings.get,
        )


class BindingsResourceWithStreamingResponse:
    def __init__(self, bindings: BindingsResource) -> None:
        self._bindings = bindings

        self.get = to_streamed_response_wrapper(
            bindings.get,
        )


class AsyncBindingsResourceWithStreamingResponse:
    def __init__(self, bindings: AsyncBindingsResource) -> None:
        self._bindings = bindings

        self.get = async_to_streamed_response_wrapper(
            bindings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["TraceGetResponse", "Inbound", "InboundLine", "Outbound", "OutboundLine"]


class InboundLine(BaseModel):
    lineno: Optional[int] = None
    """Line number in the trace log"""

    logged_at: Optional[datetime] = None

    message: Optional[str] = None

    ts: Optional[str] = None
    """Deprecated, use `logged_at` instead. End of life: November 1, 2026."""


class Inbound(BaseModel):
    lines: Optional[List[InboundLine]] = None

    pending: Optional[bool] = None


class OutboundLine(BaseModel):
    lineno: Optional[int] = None
    """Line number in the trace log"""

    logged_at: Optional[datetime] = None

    message: Optional[str] = None

    ts: Optional[str] = None
    """Deprecated, use `logged_at` instead. End of life: November 1, 2026."""


class Outbound(BaseModel):
    lines: Optional[List[OutboundLine]] = None

    pending: Optional[bool] = None


class TraceGetResponse(BaseModel):
    inbound: Inbound

    outbound: Outbound



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .namespaces import (
    NamespacesResource,
    AsyncNamespacesResource,
    NamespacesResourceWithRawResponse,
    AsyncNamespacesResourceWithRawResponse,
    NamespacesResourceWithStreamingResponse,
    AsyncNamespacesResourceWithStreamingResponse,
)
from .durable_objects import (
    DurableObjectsResource,
    AsyncDurableObjectsResource,
    DurableObjectsResourceWithRawResponse,
    AsyncDurableObjectsResourceWithRawResponse,
    DurableObjectsResourceWithStreamingResponse,
    AsyncDurableObjectsResourceWithStreamingResponse,
)

__all__ = [
    "NamespacesResource",
    "AsyncNamespacesResource",
    "NamespacesResourceWithRawResponse",
    "AsyncNamespacesResourceWithRawResponse",
    "NamespacesResourceWithStreamingResponse",
    "AsyncNamespacesResourceWithStreamingResponse",
    "DurableObjectsResource",
    "AsyncDurableObjectsResource",
    "DurableObjectsResourceWithRawResponse",
    "AsyncDurableObjectsResourceWithRawResponse",
    "DurableObjectsResourceWithStreamingResponse",
    "AsyncDurableObjectsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .user_get_params import UserGetParams as UserGetParams
from .user_edit_params import UserEditParams as UserEditParams
from .user_get_response import UserGetResponse as UserGetResponse
from .managed_get_params import ManagedGetParams as ManagedGetParams
from .user_edit_response import UserEditResponse as UserEditResponse
from .user_update_params import UserUpdateParams as UserUpdateParams
from .managed_get_response import ManagedGetResponse as ManagedGetResponse
from .user_delete_response import UserDeleteResponse as UserDeleteResponse
from .user_update_response import UserUpdateResponse as UserUpdateResponse
from .user_bulk_create_params import UserBulkCreateParams as UserBulkCreateParams
from .user_bulk_create_response import UserBulkCreateResponse as UserBulkCreateResponse
from .user_bulk_delete_response import UserBulkDeleteResponse as UserBulkDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr
from .associated_hostnames import AssociatedHostnames

__all__ = ["CertificateCreateParams"]


class CertificateCreateParams(TypedDict, total=False):
    certificate: Required[str]
    """The certificate content."""

    name: Required[str]
    """The name of the certificate."""

    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    associated_hostnames: SequenceNotStr[AssociatedHostnames]
    """The hostnames of the applications that will use this certificate."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["VersionListParams"]


class VersionListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    deployable: bool
    """Only return versions that can be used in a deployment. Ignores pagination."""

    page: int
    """Current page."""

    per_page: int
    """Items per-page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SSOUpdateParams"]


class SSOUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    enabled: bool
    """SSO Connector enabled state"""

    use_fedramp_language: bool
    """Controls the display of FedRAMP language to the user during SSO login"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.addressing.prefixes import bgp_prefix_edit_params, bgp_prefix_create_params
from ....types.addressing.prefixes.bgp_prefix import BGPPrefix

__all__ = ["BGPPrefixesResource", "AsyncBGPPrefixesResource"]


class BGPPrefixesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BGPPrefixesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BGPPrefixesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BGPPrefixesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BGPPrefixesResourceWithStreamingResponse(self)

    def create(
        self,
        prefix_id: str,
        *,
        account_id: str,
        cidr: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BGPPrefix]:
        """
        Create a BGP prefix, controlling the BGP advertisement status of a specific
        subnet. When created, BGP prefixes are initially withdrawn, and can be
        advertised with the Update BGP Prefix API.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          cidr: IP Prefix in Classless Inter-Domain Routing format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/prefixes",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            body=maybe_transform({"cidr": cidr}, bgp_prefix_create_params.BGPPrefixCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[BGPPrefix]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[BGPPrefix]], ResultWrapper[BGPPrefix]),
        )

    def list(
        self,
        prefix_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[BGPPrefix]:
        """List all BGP Prefixes within the specified IP Prefix.

        BGP Prefixes are used to
        control which specific subnets are advertised to the Internet. It is possible to
        advertise subnets more specific than an IP Prefix by creating more specific BGP
        Prefixes.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/prefixes",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            page=SyncSinglePage[BGPPrefix],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=BGPPrefix,
        )

    def edit(
        self,
        bgp_prefix_id: str,
        *,
        account_id: str,
        prefix_id: str,
        asn_prepend_count: int | Omit = omit,
        auto_advertise_withdraw: bool | Omit = omit,
        on_demand: bgp_prefix_edit_params.OnDemand | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BGPPrefix]:
        """
        Update the properties of a BGP Prefix, such as the on demand advertisement
        status (advertised or withdrawn).

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          bgp_prefix_id: Identifier of BGP Prefix.

          asn_prepend_count: Number of times to prepend the Cloudflare ASN to the BGP AS-Path attribute

          auto_advertise_withdraw: Determines if Cloudflare advertises a BYOIP BGP prefix even when there is no
              matching BGP prefix in the Magic routing table. When true, Cloudflare will
              automatically withdraw the BGP prefix when there are no matching BGP routes, and
              will resume advertising when there is at least one matching BGP route.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        if not bgp_prefix_id:
            raise ValueError(f"Expected a non-empty value for `bgp_prefix_id` but received {bgp_prefix_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/prefixes/{bgp_prefix_id}",
                account_id=account_id,
                prefix_id=prefix_id,
                bgp_prefix_id=bgp_prefix_id,
            ),
            body=maybe_transform(
                {
                    "asn_prepend_count": asn_prepend_count,
                    "auto_advertise_withdraw": auto_advertise_withdraw,
                    "on_demand": on_demand,
                },
                bgp_prefix_edit_params.BGPPrefixEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[BGPPrefix]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[BGPPrefix]], ResultWrapper[BGPPrefix]),
        )

    def get(
        self,
        bgp_prefix_id: str,
        *,
        account_id: str,
        prefix_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BGPPrefix]:
        """
        Retrieve a single BGP Prefix according to its identifier

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          bgp_prefix_id: Identifier of BGP Prefix.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        if not bgp_prefix_id:
            raise ValueError(f"Expected a non-empty value for `bgp_prefix_id` but received {bgp_prefix_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/prefixes/{bgp_prefix_id}",
                account_id=account_id,
                prefix_id=prefix_id,
                bgp_prefix_id=bgp_prefix_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[BGPPrefix]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[BGPPrefix]], ResultWrapper[BGPPrefix]),
        )


class AsyncBGPPrefixesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBGPPrefixesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBGPPrefixesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBGPPrefixesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBGPPrefixesResourceWithStreamingResponse(self)

    async def create(
        self,
        prefix_id: str,
        *,
        account_id: str,
        cidr: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BGPPrefix]:
        """
        Create a BGP prefix, controlling the BGP advertisement status of a specific
        subnet. When created, BGP prefixes are initially withdrawn, and can be
        advertised with the Update BGP Prefix API.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          cidr: IP Prefix in Classless Inter-Domain Routing format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/prefixes",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            body=await async_maybe_transform({"cidr": cidr}, bgp_prefix_create_params.BGPPrefixCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[BGPPrefix]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[BGPPrefix]], ResultWrapper[BGPPrefix]),
        )

    def list(
        self,
        prefix_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BGPPrefix, AsyncSinglePage[BGPPrefix]]:
        """List all BGP Prefixes within the specified IP Prefix.

        BGP Prefixes are used to
        control which specific subnets are advertised to the Internet. It is possible to
        advertise subnets more specific than an IP Prefix by creating more specific BGP
        Prefixes.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/prefixes",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            page=AsyncSinglePage[BGPPrefix],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=BGPPrefix,
        )

    async def edit(
        self,
        bgp_prefix_id: str,
        *,
        account_id: str,
        prefix_id: str,
        asn_prepend_count: int | Omit = omit,
        auto_advertise_withdraw: bool | Omit = omit,
        on_demand: bgp_prefix_edit_params.OnDemand | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BGPPrefix]:
        """
        Update the properties of a BGP Prefix, such as the on demand advertisement
        status (advertised or withdrawn).

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          bgp_prefix_id: Identifier of BGP Prefix.

          asn_prepend_count: Number of times to prepend the Cloudflare ASN to the BGP AS-Path attribute

          auto_advertise_withdraw: Determines if Cloudflare advertises a BYOIP BGP prefix even when there is no
              matching BGP prefix in the Magic routing table. When true, Cloudflare will
              automatically withdraw the BGP prefix when there are no matching BGP routes, and
              will resume advertising when there is at least one matching BGP route.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        if not bgp_prefix_id:
            raise ValueError(f"Expected a non-empty value for `bgp_prefix_id` but received {bgp_prefix_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/prefixes/{bgp_prefix_id}",
                account_id=account_id,
                prefix_id=prefix_id,
                bgp_prefix_id=bgp_prefix_id,
            ),
            body=await async_maybe_transform(
                {
                    "asn_prepend_count": asn_prepend_count,
                    "auto_advertise_withdraw": auto_advertise_withdraw,
                    "on_demand": on_demand,
                },
                bgp_prefix_edit_params.BGPPrefixEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[BGPPrefix]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[BGPPrefix]], ResultWrapper[BGPPrefix]),
        )

    async def get(
        self,
        bgp_prefix_id: str,
        *,
        account_id: str,
        prefix_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BGPPrefix]:
        """
        Retrieve a single BGP Prefix according to its identifier

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          bgp_prefix_id: Identifier of BGP Prefix.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        if not bgp_prefix_id:
            raise ValueError(f"Expected a non-empty value for `bgp_prefix_id` but received {bgp_prefix_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bgp/prefixes/{bgp_prefix_id}",
                account_id=account_id,
                prefix_id=prefix_id,
                bgp_prefix_id=bgp_prefix_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[BGPPrefix]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[BGPPrefix]], ResultWrapper[BGPPrefix]),
        )


class BGPPrefixesResourceWithRawResponse:
    def __init__(self, bgp_prefixes: BGPPrefixesResource) -> None:
        self._bgp_prefixes = bgp_prefixes

        self.create = to_raw_response_wrapper(
            bgp_prefixes.create,
        )
        self.list = to_raw_response_wrapper(
            bgp_prefixes.list,
        )
        self.edit = to_raw_response_wrapper(
            bgp_prefixes.edit,
        )
        self.get = to_raw_response_wrapper(
            bgp_prefixes.get,
        )


class AsyncBGPPrefixesResourceWithRawResponse:
    def __init__(self, bgp_prefixes: AsyncBGPPrefixesResource) -> None:
        self._bgp_prefixes = bgp_prefixes

        self.create = async_to_raw_response_wrapper(
            bgp_prefixes.create,
        )
        self.list = async_to_raw_response_wrapper(
            bgp_prefixes.list,
        )
        self.edit = async_to_raw_response_wrapper(
            bgp_prefixes.edit,
        )
        self.get = async_to_raw_response_wrapper(
            bgp_prefixes.get,
        )


class BGPPrefixesResourceWithStreamingResponse:
    def __init__(self, bgp_prefixes: BGPPrefixesResource) -> None:
        self._bgp_prefixes = bgp_prefixes

        self.create = to_streamed_response_wrapper(
            bgp_prefixes.create,
        )
        self.list = to_streamed_response_wrapper(
            bgp_prefixes.list,
        )
        self.edit = to_streamed_response_wrapper(
            bgp_prefixes.edit,
        )
        self.get = to_streamed_response_wrapper(
            bgp_prefixes.get,
        )


class AsyncBGPPrefixesResourceWithStreamingResponse:
    def __init__(self, bgp_prefixes: AsyncBGPPrefixesResource) -> None:
        self._bgp_prefixes = bgp_prefixes

        self.create = async_to_streamed_response_wrapper(
            bgp_prefixes.create,
        )
        self.list = async_to_streamed_response_wrapper(
            bgp_prefixes.list,
        )
        self.edit = async_to_streamed_response_wrapper(
            bgp_prefixes.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            bgp_prefixes.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["ASNRelResponse", "Meta", "Rel"]


class Meta(BaseModel):
    data_time: str

    query_time: str

    total_peers: int


class Rel(BaseModel):
    asn1: int

    asn1_country: str

    asn1_name: str

    asn2: int

    asn2_country: str

    asn2_name: str

    rel: str


class ASNRelResponse(BaseModel):
    meta: Meta

    rels: List[Rel]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["PermissionCreateResponse"]


class PermissionCreateResponse(BaseModel):
    success: Optional[bool] = None
    """Whether the update succeeded or not"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rayid import (
    RayIDResource,
    AsyncRayIDResource,
    RayIDResourceWithRawResponse,
    AsyncRayIDResourceWithRawResponse,
    RayIDResourceWithStreamingResponse,
    AsyncRayIDResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .control.control import (
    ControlResource,
    AsyncControlResource,
    ControlResourceWithRawResponse,
    AsyncControlResourceWithRawResponse,
    ControlResourceWithStreamingResponse,
    AsyncControlResourceWithStreamingResponse,
)
from .received.received import (
    ReceivedResource,
    AsyncReceivedResource,
    ReceivedResourceWithRawResponse,
    AsyncReceivedResourceWithRawResponse,
    ReceivedResourceWithStreamingResponse,
    AsyncReceivedResourceWithStreamingResponse,
)

__all__ = ["LogsResource", "AsyncLogsResource"]


class LogsResource(SyncAPIResource):
    @cached_property
    def control(self) -> ControlResource:
        return ControlResource(self._client)

    @cached_property
    def rayid(self) -> RayIDResource:
        return RayIDResource(self._client)

    @cached_property
    def received(self) -> ReceivedResource:
        return ReceivedResource(self._client)

    @cached_property
    def with_raw_response(self) -> LogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LogsResourceWithStreamingResponse(self)


class AsyncLogsResource(AsyncAPIResource):
    @cached_property
    def control(self) -> AsyncControlResource:
        return AsyncControlResource(self._client)

    @cached_property
    def rayid(self) -> AsyncRayIDResource:
        return AsyncRayIDResource(self._client)

    @cached_property
    def received(self) -> AsyncReceivedResource:
        return AsyncReceivedResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLogsResourceWithStreamingResponse(self)


class LogsResourceWithRawResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

    @cached_property
    def control(self) -> ControlResourceWithRawResponse:
        return ControlResourceWithRawResponse(self._logs.control)

    @cached_property
    def rayid(self) -> RayIDResourceWithRawResponse:
        return RayIDResourceWithRawResponse(self._logs.rayid)

    @cached_property
    def received(self) -> ReceivedResourceWithRawResponse:
        return ReceivedResourceWithRawResponse(self._logs.received)


class AsyncLogsResourceWithRawResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

    @cached_property
    def control(self) -> AsyncControlResourceWithRawResponse:
        return AsyncControlResourceWithRawResponse(self._logs.control)

    @cached_property
    def rayid(self) -> AsyncRayIDResourceWithRawResponse:
        return AsyncRayIDResourceWithRawResponse(self._logs.rayid)

    @cached_property
    def received(self) -> AsyncReceivedResourceWithRawResponse:
        return AsyncReceivedResourceWithRawResponse(self._logs.received)


class LogsResourceWithStreamingResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

    @cached_property
    def control(self) -> ControlResourceWithStreamingResponse:
        return ControlResourceWithStreamingResponse(self._logs.control)

    @cached_property
    def rayid(self) -> RayIDResourceWithStreamingResponse:
        return RayIDResourceWithStreamingResponse(self._logs.rayid)

    @cached_property
    def received(self) -> ReceivedResourceWithStreamingResponse:
        return ReceivedResourceWithStreamingResponse(self._logs.received)


class AsyncLogsResourceWithStreamingResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

    @cached_property
    def control(self) -> AsyncControlResourceWithStreamingResponse:
        return AsyncControlResourceWithStreamingResponse(self._logs.control)

    @cached_property
    def rayid(self) -> AsyncRayIDResourceWithStreamingResponse:
        return AsyncRayIDResourceWithStreamingResponse(self._logs.rayid)

    @cached_property
    def received(self) -> AsyncReceivedResourceWithStreamingResponse:
        return AsyncReceivedResourceWithStreamingResponse(self._logs.received)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["CrawlDeleteResponse"]


class CrawlDeleteResponse(BaseModel):
    job_id: str
    """The ID of the cancelled job."""

    message: str
    """Cancellation confirmation message."""
